/************************************************************************
 *   mIRC MySQL interface.cpp
 *   Copyright (C) 2000-2003 Michael Reynolds
 *
 *   See file readme.txt for additional names of the programmers.
 *
 *   This file is part of mIRC MySQL.
 *
 *   mIRC MySQL is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   mIRC MySQL is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with mIRC MySQL; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma check_stack(off)
#pragma comment(linker,"/OPT:NOWIN98")
#include "mircmysql.h"

/************************************************************************
 * Non MySQL functions                                                  *
 ************************************************************************/

mIRC(Echo) {
	char *data=strdup(mIRCdata);
	if (data==NULL || *data=='\0') {
		mIRCdata="0 030 Echo";
	}
	else {
		snprintf(mIRCdata,BUFSIZE,"You supplied: %s",data);
	}
	free(data);
	return 3;
}

mIRC(Reverse) {
	char *data=strdup(mIRCdata);
	snprintf(mIRCdata,BUFSIZE,strrev(data));
	free(data);
	return 3;
}

mIRC(DLLInfo) {
	char *data=strdup(mIRCdata);
#if !defined(STATUS) || STATUS==ALPHA || STATUS==BETA || STATUS==RC
	snprintf(mIRCdata,BUFSIZE,"mIRC MySQL v%d.%d.%.2d(%s) for libmysql v%s by wshs. [\37\2NO SUPPORT FOR THIS VERSION\2\37\17]",VERSIONmajor,VERSIONminor,VERSIONpatch,bench,mysql_get_client_info());
#else
	snprintf(mIRCdata,BUFSIZE,"mIRC MySQL v%d.%d.%.2d(%s) for libmysql v%s by wshs.",VERSIONmajor,VERSIONminor,VERSIONpath,bench,mysql_get_client_info());
#endif
	free(data);
	return 3;
}

mIRC(ToBase64) {
	char *data=strdup(mIRCdata);
	unsigned long i=0;
	if (data==NULL || *data=='\0')
		mIRCdata="0 030 ToBase64";
	else {
		i=strtoul(data,NULL,10);
		snprintf(mIRCdata,BUFSIZE,"1 026 ToBase64 %s",int_to_base64(i));
	}
	free(data);
	return 3;
}

mIRC(ToBase10) {
	char *data=strdup(mIRCdata);
	unsigned long i=0;
	if (data==NULL || *data=='\0')
		mIRCdata="0 030 ToBase10";
	else {
		i=base64_to_int(data);
		snprintf(mIRCdata,BUFSIZE,"1 026 ToBase10 %lu",i);
	}
	free(data);
	return 3;
}

/************************************************************************
 * MySQL functions                                                      *
 ************************************************************************/

mIRC(Connect) {
	char *data=strdup(mIRCdata),*lbl=NULL,*input_word_1=NULL,*sql_host=NULL,*sql_db=NULL,*sql_user=NULL,*sql_pass=NULL,*timeout=NULL;
	const char *sqlerror;
	lbl=strtok(data," "),input_word_1=strtok(NULL," ");
	int sqlerrno=0,sql_port=MYSQL_PORT,ssl=0,compress=0,flags=0;
	bool error=false;
	LABELS *templabel=NULL;
	if (check_version(1)) {
		mIRCdata="";
		error=true;
	}
	else if (lbl==NULL || *lbl=='\0') {
		mIRCdata="0 001 Connect";
		error=true;
	}
	else if (strlen(lbl) > LBLMAX) {
		snprintf(mIRCdata,BUFSIZE,"0 002 Connect %s",lbl);
		error=true;
	}
	else if (atoi(lbl) != 0) {
		snprintf(mIRCdata,BUFSIZE,"0 003 Connect %s",lbl);
		error=true;
	}
	else if (findlabel(lbl) != NULL) {
		snprintf(mIRCdata,BUFSIZE,"0 004 Connect %s",lbl);
		error=true;
	}
	else {
		if (input_word_1 && *input_word_1 && (*input_word_1 == '-')) {
			while (*input_word_1++ && !error) {
				switch (*input_word_1) {
				case 'h':
					sql_host=strtok(NULL," ");
					if (sql_host==NULL || *sql_host=='\0') {
						mIRCdata="0 005 Connect";
						error=true;
					}
					break;
				case 'p':
					sql_port=atoi(strtok(NULL," "));
					if (!sql_port) {
						mIRCdata="0 006 Connect";
						error=true;
					}
					break;
				case 'u':
					sql_user=strtok(NULL," ");
					if (sql_user==NULL || *sql_user=='\0') {
						mIRCdata="0 007 Connect";
						error=true;
					}
					break;
				case 'w':
					sql_pass=strtok(NULL," ");
					if (sql_pass==NULL || *sql_pass=='\0') {
						mIRCdata="0 008 Connect";
						error=true;
					}
					break;
				case 'd':
					sql_db=strtok(NULL," ");
					if (sql_db==NULL || *sql_db=='\0') {
						mIRCdata="0 019 Connect";
						error=true;
					}
					break;
				case 't':
					timeout=strtok(NULL," ");
					if (atoi(timeout)==0) {
						mIRCdata="0 032 Connect";
						error=true;
					}
					break;
				case 's':
					ssl=1;
					flags|=CLIENT_SSL;
					break;
				case 'c':
					compress=1;
					flags|=CLIENT_COMPRESS;
					break;
				default:
					if (*input_word_1 && (*input_word_1 > 0) && (*input_word_1 != 32)) {
						snprintf(mIRCdata,BUFSIZE,"0 009 Connect %c",*input_word_1);
						error=true;
					}
					break;
				}
			}
		}
	}
	if (!error) {
		templabel=makelabel(lbl);
		mysql_init(&templabel->mysql);
		if (mysql_real_connect(&templabel->mysql,sql_host,sql_user,sql_pass,sql_db,sql_port,NULL,ssl|compress)) {
			if (timeout)
				mysql_options(&templabel->mysql,MYSQL_OPT_CONNECT_TIMEOUT,timeout);
			snprintf(mIRCdata,BUFSIZE,"1 010 Connect %s",lbl);
		}
		else {
			sqlerror=mysql_error(&templabel->mysql);
			sqlerrno=mysql_errno(&templabel->mysql);
			snprintf(mIRCdata,BUFSIZE,"0 011 Connect %d %s",sqlerrno,sqlerror);
		}
	}
	free(data);
	return 3;
}

mIRC(Close) {
	char *data=strdup(mIRCdata),*lbl=NULL;
	lbl=strtok(data," ");
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1))
		mIRCdata="";
	else if (lbl==NULL || *lbl=='\0')
		snprintf(mIRCdata,BUFSIZE,"0 001 Close");
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 Close %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 Close %s",lbl);
	else {
		removelabel(templabel);
		snprintf(mIRCdata,BUFSIZE,"1 013 Close %s",lbl);
	}
	free(data);
	return 3;
}

mIRC(Labels) {
	char *data=strdup(mIRCdata);
	unsigned long lbl=0;
	LABELS *templabel=NULL;
	lbl=atoi(strtok(data," "));
	if (check_version(1))
		mIRCdata="";
	else if (lbl > labelcount())
		mIRCdata="0 014 Labels";
	else if (lbl < 0)
		mIRCdata="0 015 Labels";
	else if (lbl == 0)
		snprintf(mIRCdata,BUFSIZE,"1 016 Labels %d",labelcount());
	else {
		templabel=findlabel_by_digit(lbl);
		snprintf(mIRCdata,BUFSIZE,"1 016 Labels %s",templabel->label);
	}
	free(data);
	return 3;
}

mIRC(RowsAffected) {
	char *data=strdup(mIRCdata),*lbl=NULL;
	lbl=strtok(data," ");
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1))
		mIRCdata="";
	else if (lbl==NULL || *lbl=='\0')
		snprintf(mIRCdata,BUFSIZE,"0 001 RowsAffected");
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 RowsAffected %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 RowsAffected %s",lbl);
	else if ((int)mysql_affected_rows(&templabel->mysql) == -1)
		snprintf(mIRCdata,BUFSIZE,"1 017 RowsAffected %s 0",lbl);
	else {
		snprintf(mIRCdata,BUFSIZE,"1 017 RowsAffected %s %I64u",templabel->label,(unsigned __int64) mysql_affected_rows(&templabel->mysql));
	}
	free(data);
	return 3;
}

mIRC(Rows) {
	char *data=strdup(mIRCdata),*lbl=NULL;
	lbl=strtok(data," ");
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1))
		mIRCdata="";
	else if (lbl==NULL || *lbl=='\0')
		snprintf(mIRCdata,BUFSIZE,"0 001 Rows");
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 Rows %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 Rows %s",lbl);
	else
		if (templabel->result == NULL)
			snprintf(mIRCdata,BUFSIZE,"0 028 Rows %s",lbl);
		else
			snprintf(mIRCdata,BUFSIZE,"1 018 Rows %s %I64u",templabel->label,(unsigned __int64) mysql_num_rows(templabel->result));
	free(data);
	return 3;
}

mIRC(Query) {
	char *data=strdup(mIRCdata),*lbl=NULL,*flags=NULL,*sql_db=NULL,*buf=NULL;
	const char *flagpoint=flags;
	int i=0;
	bool error=false;
	lbl=strtok(data," "),buf=strtok(NULL,""),sql_db="";
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1)) {
		mIRCdata="";
		error=true;
	}
	if (lbl==NULL || *lbl=='\0') {
		mIRCdata="0 001 Query";
		error=true;
	}
	else if (strlen(lbl) > LBLMAX) {
		snprintf(mIRCdata,BUFSIZE,"0 002 Query %s",lbl);
		error=true;
	}
	else if (templabel == NULL) {
		snprintf(mIRCdata,BUFSIZE,"0 012 Query %s",lbl);
		error=true;
	}
	else if (buf==NULL || *buf=='\0') {
		mIRCdata="0 030 Query";
		error=true;
	}
	else if (*buf == '-') {
		flags=strtok(buf," "),flagpoint=flags;
		while (*flagpoint++ && !error) {
			switch (*flagpoint) {
			case 'd':
				sql_db=strtok(NULL," ");
				if (sql_db==NULL || *sql_db=='\0') {
					mIRCdata="0 019 Query";
					error=true;
				}
				break;
			default:
				if ((*flagpoint) && (*flagpoint > 0) && (*flagpoint != 32)) {
					snprintf(mIRCdata,BUFSIZE,"0 009 Query %c",*flagpoint);
					error=true;
				}
				break;
			}
		}
		buf=strtok(NULL,"");
		if (buf==NULL || *buf=='\0') {
			mIRCdata="0 030 Query";
			error=true;
		}
	}
	if (!error) {
		if (mysql_ping(&templabel->mysql)) {
			mIRCdata="0 034 Query";
			error=true;
		}
		if (templabel->result != NULL)
			mysql_free_result(templabel->result);
		if ((sql_db!=NULL) && (*sql_db!='\0') && mysql_select_db(&templabel->mysql,sql_db)) {
			snprintf(mIRCdata,BUFSIZE,"0 020 Query %s",sql_db);
			error=true;
		}
		else if (mysql_real_query(&templabel->mysql,buf,strlen(buf))) {
			snprintf(mIRCdata,BUFSIZE,"0 021 Query %s",buf);
			error=true;
		}
		if (!error) {
			templabel->result=mysql_store_result(&templabel->mysql);
			if (templabel->result != NULL)
				i=1;
			snprintf(mIRCdata,BUFSIZE,"1 022 Query %s %d",templabel->label,i);
		}
	}
	free(data);
	return 3;
}

mIRC(Fields) {
	char *data=strdup(mIRCdata),*lbl=NULL,*buf=(char*)malloc(BUFSIZE);
	lbl=strtok(data," ");
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	my_ulonglong i=0, num_fields=0;
	if (check_version(1))
		mIRCdata="";
	else if (lbl==NULL || *lbl=='\0')
		mIRCdata="0 001 Fields";
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 Fields %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 Fields %s",lbl);
	else if (templabel->result == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 028 Fields %s",lbl);
	else {
		MYSQL_FIELD *fields;
		num_fields=mysql_num_fields(templabel->result);
		fields=mysql_fetch_fields(templabel->result);
		*buf=0;
		for (i=0;i<num_fields;i++) {
			if (*buf)
				buf=strcat(buf,",");
			m_strncat(buf,fields[i].name,BUFSIZE);
		}
		if (*buf)
			snprintf(mIRCdata,BUFSIZE,"1 023 Fields %s %I64u %s",templabel->label,(unsigned __int64)num_fields,buf);
		else
			snprintf(mIRCdata,BUFSIZE,"1 023 Fields %s %I64u",templabel->label,(unsigned __int64)num_fields);
	}
	free(data);
	free(buf);
	return 3;
}

mIRC(FetchRow) {
	char *data=strdup(mIRCdata),*lbl=NULL,*rw=NULL,*sep=NULL,*buf=(char*)malloc(BUFSIZE),*buf2=(char*)malloc(BUFSIZE);
	lbl=strtok(data," "),rw=strtok(NULL," "),sep=strtok(NULL," "),*buf=0,*buf2=0;
	LABELS *templabel=NULL;
	MYSQL_ROW rowdata;
	unsigned long *lengths=0;
	bool error=false;
	my_ulonglong i=0,num_fields=0,row=0;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1)) {
		mIRCdata="";
		error=true;
	}
	else if (lbl==NULL || *lbl=='\0') {
		mIRCdata="0 001 FetchRow";
		error=true;
	}
	else if (rw==NULL || *rw=='\0') {
		mIRCdata="0 024 FetchRow";
		error=true;
	}
	else
		row=strtoul(rw,NULL,10);
	if (error)
		;
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 FetchRow %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 FetchRow %s",lbl);
	else if (templabel->result == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 028 FetchRow %s",lbl);
	else if (!row || (row < 0) || (row > mysql_num_rows(templabel->result)))
		snprintf(mIRCdata,BUFSIZE,"0 024 FetchRow %s %d",lbl,row);
	else {
		if (sep==NULL || *sep=='\0' || (strlen(sep) != 1))
			sep="	";
		mysql_data_seek(templabel->result,(row - 1));
		num_fields=mysql_num_fields(templabel->result);
		rowdata = mysql_fetch_row(templabel->result);
		if (rowdata==NULL)
			snprintf(mIRCdata,BUFSIZE,"0 031 FetchRow %s %d",lbl,row);
		else {
			lengths=mysql_fetch_lengths(templabel->result);
			for (i=0;i<num_fields;i++) {
				if (*buf)
					m_strncat(buf,sep,BUFSIZE);
				snprintf(buf2,BUFSIZE,"%.*s",lengths[i], (!rowdata[i] || rowdata[i] == NULL) ? "NULL" : rowdata[i]);
				if (buf2 && *buf2)
					m_strncat(buf,buf2,BUFSIZE);
			}
			snprintf(mIRCdata,BUFSIZE,"1 025 FetchRow %s %I64u %s",templabel->label,(unsigned __int64)num_fields,buf);
		}
	}
	free(data);
	free(buf);
	free(buf2);
	return 3;
}

mIRC(EscapeString) {
	char *data=strdup(mIRCdata),*buf,query[2048]="";
	if (check_version(1))
		mIRCdata="";
	else if (data==NULL || *data=='\0')
		mIRCdata="1 027 EscapeString";
	else {
		buf=strmov(query,"");
		buf+=mysql_escape_string(buf,data,strlen(data));
		snprintf(mIRCdata,BUFSIZE,"1 027 EscapeString %d %s",(unsigned int) (buf - query),query);
	}
	free(data);
	free(query);
	return 3;
}

mIRC(bEscapeString) {
	char *data=strdup(mIRCdata),*buf,query[2048]="";
	if (check_version(1))
		mIRCdata="";
	else if (data==NULL || *data=='\0')
		mIRCdata="";
	else {
		buf=strmov(query,"");
		buf+=mysql_escape_string(buf,data,strlen(data));
		mIRCdata=query;
	}
	free(data);
	free(query);
	return 3;
}

mIRC(SetDatabase) {
	char *data=strdup(mIRCdata),*lbl=NULL,*sql_db=NULL;
	lbl=strtok(data," "),sql_db=strtok(NULL," ");
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1))
		mIRCdata="";
	else if (lbl==NULL || *lbl=='\0')
		snprintf(mIRCdata,BUFSIZE,"0 001 SetDatabase");
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 SetDatabase %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 SetDatabase %s",lbl);
	else if (sql_db==NULL || *sql_db=='\0')
		snprintf(mIRCdata,BUFSIZE,"0 019 SetDatabase");
	else if (mysql_ping(&templabel->mysql))
		snprintf(mIRCdata,BUFSIZE,"0 034 SetDatabase");
	else if (mysql_select_db(&templabel->mysql,sql_db))
		snprintf(mIRCdata,BUFSIZE,"0 020 SetDatabase %s %s",templabel->label,sql_db);
	else
		snprintf(mIRCdata,BUFSIZE,"1 022 SetDatabase %s %s",templabel->label,sql_db);
	free(data);
	return 3;
}

mIRC(Error) {
	char *data=strdup(mIRCdata),*lbl=NULL;
	const char *sqlerror;
	lbl=strtok(data," ");
	int sqlerrno=0;
	LABELS *templabel=NULL;
	if (lbl)
		templabel=findlabel(lbl);
	if (check_version(1))
		mIRCdata="";
	else if (lbl==NULL || *lbl=='\0')
		mIRCdata="0 001 Error";
	else if (strlen(lbl) > LBLMAX)
		snprintf(mIRCdata,BUFSIZE,"0 002 Error %s",lbl);
	else if (templabel == NULL)
		snprintf(mIRCdata,BUFSIZE,"0 012 Error %s",lbl);
	else if (mysql_ping(&templabel->mysql))
		mIRCdata="0 034 Error";
	else {
		sqlerror=mysql_error(&templabel->mysql);
		sqlerrno=mysql_errno(&templabel->mysql);
		snprintf(mIRCdata,BUFSIZE,"1 029 Error %d %s",sqlerrno,sqlerror);
	}
	free(data);
	return 3;
}
