/************************************************************************
 *   mIRC MySQL labels.cpp
 *   Copyright (C) 2000-2003 Michael Reynolds
 *
 *   See file readme.txt for additional names of the programmers.
 *
 *   This file is part of mIRC MySQL.
 *
 *   mIRC MySQL is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   mIRC MySQL is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with mIRC MySQL; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma check_stack(off)
#pragma comment(linker,"/OPT:NOWIN98")
#include "mircmysql.h"
#include <malloc.h>

void freelabel(LABELS *label) {
	free(label->label);
	if (&label->mysql!=NULL)
		mysql_close(&label->mysql);
	if (label->result!=NULL)
		mysql_free_result(label->result);
	free(label);
}

LABELS *findlabel(const char *lbl) {
	LABELS *temp;
	if (lbl==NULL)
		return NULL;
	for (temp=mainlabel;temp;temp=temp->next) {
		if (!stricmp(temp->label,lbl))
			return temp;
	}
	return NULL;
}

LABELS *findlabel_by_digit(unsigned long lbl) {
	LABELS *temp;
	unsigned long i;
	for (temp=mainlabel,i=1;temp!=NULL;temp=temp->next,i++)
		if ((i == lbl) && (lbl != 0))
			return temp;
	return NULL;
}

unsigned long labelcount() {
	LABELS *temp;
	unsigned long i=0;
	for (temp=mainlabel,i=0;temp!=NULL;temp=temp->next,i++)
		;
	return i;
}

unsigned int field_to_number(MYSQL_RES data,char *fieldname) {
	unsigned int i=0,num_fields=0;
	MYSQL_FIELD *fields;
	num_fields=mysql_num_fields(&data);
	fields = mysql_fetch_fields(&data);
	for (i=0;i<num_fields;i++) {
		if (!stricmp(fieldname,fields[i].name))
			return i;
	}
	return 0;
}

void removelabel(LABELS *lbl) {
	if (lbl->next)
		lbl->next->prev = lbl->prev;
	if (lbl->prev)
		lbl->prev->next = lbl->next;
	else
		mainlabel = lbl->next;
	lbl->next=NULL;
	lbl->prev=NULL;
	freelabel(lbl);
}

void insertlabel(LABELS *lbl) {
	LABELS *lbl2,*lbl3;
    for (lbl3=NULL,lbl2=mainlabel;lbl2;lbl3=lbl2,lbl2=lbl2->next)
		;
    lbl->prev = lbl3;
	lbl->next = lbl2;
    if (!lbl3) {
		mainlabel = lbl;
		mainlabel->prev=NULL;
		mainlabel->next=NULL;
	}
    else {
		lbl3->next = lbl;
	}
	if (lbl2) {
		lbl2->prev = lbl;
	}
	else {
		lbl->next=NULL;
	}
}

LABELS *makelabel(char *lbl) {
	LABELS *newlbl=initlabel();
	strncpy(newlbl->label,lbl,LBLMAX);
	insertlabel(newlbl);
	return newlbl;
}

LABELS *initlabel() {
	LABELS *newlbl=(LABELS*)malloc((sizeof LABELS));
	newlbl->label=(char*)malloc(LBLMAX+1);
	mysql_init(&newlbl->mysql);
	newlbl->result=NULL;
	newlbl->next=NULL;
	newlbl->prev=NULL;
	return newlbl;
}
